"use strict";

var _Object$defineProperty = require("@babel/runtime-corejs3/core-js-stable/object/define-property");

var _interopRequireDefault = require("@babel/runtime-corejs3/helpers/interopRequireDefault");

_Object$defineProperty(exports, "__esModule", {
  value: true
});

exports["default"] = exports.QueryBuilder = void 0;

var _classCallCheck2 = _interopRequireDefault(require("@babel/runtime-corejs3/helpers/classCallCheck"));

var _createClass2 = _interopRequireDefault(require("@babel/runtime-corejs3/helpers/createClass"));

var _isArray = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/array/is-array"));

var _map = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/map"));

var _forEach = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/for-each"));

var _keys = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/object/keys"));

var _indexOf = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/index-of"));

var _endsWith = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/ends-with"));

var _concat = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/concat"));

var _filter = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/filter"));

var _sort = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/sort"));

var _trim = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/trim"));

var _parseInt2 = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/parse-int"));

var _startsWith = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/starts-with"));

var _validators = _interopRequireDefault(require("./validators"));

/* eslint no-undefined : 0 */
function buildFilters(queryBuilder, filterSet) {
  var filters = [],
      getMatch = function getMatch(field, value, isPhrase) {
    var filter = {
      match: {}
    };
    isPhrase = isPhrase || false;
    filter.match[field] = {
      query: value
    };

    if (isPhrase) {
      filter.match[field].type = 'phrase';
    } else {
      filter.match[field].operator = 'and';
    }

    return filter;
  },
      getMatches = function getMatches(field, value) {
    if (typeof value === 'string' && /\,/g.test(value)) {
      value = value.split(/\,/g);
    }

    if ((0, _isArray["default"])(value)) {
      return (0, _map["default"])(value).call(value, function (item) {
        return getMatch(field, item);
      });
    }

    return [getMatch(field, value)];
  },
      getRange = function getRange(field, value) {
    var filter = {
      range: {}
    };
    filter.range[field] = value;
    return filter;
  },
      getTerm = function getTerm(field, value) {
    var filter = {
      term: {}
    };
    filter.term[field] = value;
    return filter;
  },
      getTerms = function getTerms(field, value) {
    if (typeof value === 'string' && /\,/g.test(value)) {
      value = value.split(/\,/g);
    }

    if ((0, _isArray["default"])(value)) {
      return (0, _map["default"])(value).call(value, function (item) {
        return getTerm(field, item);
      });
    }

    return [getTerm(field, value)];
  },
      gt,
      gte,
      lt,
      lte,
      missingFilter;

  if (filterSet.beginsWith) {
    var _context;

    (0, _forEach["default"])(_context = (0, _keys["default"])(filterSet.beginsWith)).call(_context, function (field) {
      var _context2;

      if ((0, _indexOf["default"])(_context2 = queryBuilder.matchFields).call(_context2, field) >= 0) {
        filters.push(getMatch(field, [filterSet.beginsWith[field]].join('')));
        return;
      }

      filters.push(getTerm(field, [filterSet.beginsWith[field], '*'].join('')));
    });
  }

  if (filterSet.contains) {
    var _context3;

    (0, _forEach["default"])(_context3 = (0, _keys["default"])(filterSet.contains)).call(_context3, function (field) {
      var _context4;

      if ((0, _indexOf["default"])(_context4 = queryBuilder.matchFields).call(_context4, field) >= 0) {
        filters.push(getMatch(field, [filterSet.contains[field]].join('')));
        return;
      }

      filters.push(getTerm(field, ['*', filterSet.contains[field], '*'].join('')));
    });
  }

  if ((0, _endsWith["default"])(filterSet)) {
    var _context5;

    (0, _forEach["default"])(_context5 = (0, _keys["default"])((0, _endsWith["default"])(filterSet))).call(_context5, function (field) {
      var _context6;

      if ((0, _indexOf["default"])(_context6 = queryBuilder.matchFields).call(_context6, field) >= 0) {
        filters.push(getMatch(field, [(0, _endsWith["default"])(filterSet)[field]].join('')));
        return;
      }

      filters.push(getTerm(field, ['*', (0, _endsWith["default"])(filterSet)[field]].join('')));
    });
  }

  if (filterSet.exact) {
    var _context7;

    (0, _forEach["default"])(_context7 = (0, _keys["default"])(filterSet.exact)).call(_context7, function (field) {
      var _context8;

      if ((0, _indexOf["default"])(_context8 = queryBuilder.matchFields).call(_context8, field) >= 0) {
        filters = (0, _concat["default"])(filters).call(filters, getMatches(field, [filterSet.exact[field]].join(''), true));
        return;
      }

      filters = (0, _concat["default"])(filters).call(filters, getTerms(field, filterSet.exact[field]));
    });
  } // handle checks for the existence of a field


  if (filterSet.exists) {
    var _context9;

    if (typeof filterSet.exists === 'string' && /\,/g.test(filterSet.exists)) {
      filterSet.exists = filterSet.exists.split(/\,/g);
    }

    if (!(0, _isArray["default"])(filterSet.exists)) {
      filterSet.exists = [filterSet.exists];
    }

    (0, _forEach["default"])(_context9 = filterSet.exists).call(_context9, function (field) {
      var existsFilter = {
        exists: {
          field: field
        }
      };
      filters.push(existsFilter);
    });
  }

  if (filterSet.greaterThan || filterSet.gt) {
    var _context10;

    gt = filterSet.greaterThan || filterSet.gt;
    (0, _forEach["default"])(_context10 = (0, _keys["default"])(gt)).call(_context10, function (field) {
      filters.push(getRange(field, {
        gt: gt[field]
      }));
    });
  }

  if (filterSet.greaterThanEqual || filterSet.gte) {
    var _context11;

    gte = filterSet.greaterThanEqual || filterSet.gte;
    (0, _forEach["default"])(_context11 = (0, _keys["default"])(gte)).call(_context11, function (field) {
      filters.push(getRange(field, {
        gte: gte[field]
      }));
    });
  }

  if (filterSet.lessThan || filterSet.lt) {
    var _context12;

    lt = filterSet.lessThan || filterSet.lt;
    (0, _forEach["default"])(_context12 = (0, _keys["default"])(lt)).call(_context12, function (field) {
      filters.push(getRange(field, {
        lt: lt[field]
      }));
    });
  }

  if (filterSet.lessThanEqual || filterSet.lte) {
    var _context13;

    lte = filterSet.lessThanEqual || filterSet.lte;
    (0, _forEach["default"])(_context13 = (0, _keys["default"])(lte)).call(_context13, function (field) {
      filters.push(getRange(field, {
        lte: lte[field]
      }));
    });
  } // handle checks for the non-existence of a field


  if (filterSet.missing) {
    var _context14;

    if (typeof filterSet.missing === 'string' && /\,/g.test(filterSet.missing)) {
      filterSet.missing = filterSet.missing.split(/\,/g);
    }

    if (!(0, _isArray["default"])(filterSet.missing)) {
      filterSet.missing = [filterSet.missing];
    }

    (0, _forEach["default"])(_context14 = filterSet.missing).call(_context14, function (field) {
      missingFilter = {
        missing: {
          exists: {
            field: field
          }
        }
      };
      filters.push(missingFilter);
    });
  }

  if (filterSet.ne) {
    var _context15;

    (0, _forEach["default"])(_context15 = (0, _keys["default"])(filterSet.ne)).call(_context15, function (field) {
      filters.push(getMatch(field, [filterSet.ne[field]].join('')));
    });
  } // return the array of filters to the caller


  return filters;
}

var QueryBuilder = /*#__PURE__*/function () {
  function QueryBuilder(matchFields, diagnosticsFilterKey) {
    (0, _classCallCheck2["default"])(this, QueryBuilder);
    this.matchFields = matchFields;
    this.diagnosticsFilterKey = diagnosticsFilterKey;
  }

  (0, _createClass2["default"])(QueryBuilder, [{
    key: "buildQuery",
    value: function buildQuery(queryOptions) {
      var _this = this;

      var defaultQuery = {
        'match_all': {}
      },
          filter,
          filters = queryOptions.filters,
          hasFilterFilters = false,
          hasQueryFilters = false,
          mustNot,
          query = {
        query: {
          bool: {}
        }
      },
          queryBool,
          sortFields,
          sortOptions = []; // reference bool for simplicity

      queryBool = query.query.bool; // keyword search

      if (filters && filters.keyword) {
        var _context16;

        // adjusted keyword filter for ES 6.x and up
        filter = {
          'multi_match': {
            query: filters.keyword
          }
        };
        queryBool.must = (0, _concat["default"])(_context16 = queryBool.must || []).call(_context16, filter);
      } // add analytics filters to the query


      if (filters && filters.diagnostics && this.diagnosticsFilterKey) {
        var _context17;

        if (!(0, _isArray["default"])(filters.diagnostics)) {
          if (/\,/g.test(filters.diagnostics)) {
            filters.diagnostics = filters.diagnostics.split(/\,/);
          } else {
            filters.diagnostics = [filters.diagnostics];
          }
        }

        queryBool.filter = (0, _filter["default"])(queryBool) || {};
        (0, _forEach["default"])(_context17 = filters.diagnostics).call(_context17, function (stat) {
          if (typeof _this.diagnosticsFilterKey[stat] !== 'undefined') {
            var _context18;

            (0, _filter["default"])(queryBool).terms = (0, _concat["default"])(_context18 = (0, _filter["default"])(queryBool).terms || []).call(_context18, _this.diagnosticsFilterKey[stat]());
          }
        });
      } // add mandatory filters to the query


      if (filters && filters.mandatory) {
        var _context21;

        // notEqual queries need to be processed differently than normal
        // match queries
        if (filters.mandatory.ne || filters.mandatory.notEqual || filters.mandatory.notEqualTo) {
          var _context19;

          mustNot = {
            'ne': filters.mandatory.ne || filters.mandatory.notEqual || filters.mandatory.notEqualTo
          }; // remove notEqual parameters so they aren't processed again

          filters.mandatory.ne = undefined;
          filters.mandatory.notEqual = undefined;
          filters.mandatory.notEqualTo = undefined;
          (0, _forEach["default"])(_context19 = buildFilters(this, mustNot)).call(_context19, function (filter) {
            // right now, only exact match is support
            if (filter.match) {
              var _context20;

              queryBool['must_not'] = (0, _concat["default"])(_context20 = queryBool['must_not'] || []).call(_context20, filter);
              return;
            }
          });
        }

        (0, _forEach["default"])(_context21 = buildFilters(this, filters.mandatory)).call(_context21, function (filter) {
          var _context24;

          if (filter.match) {
            var _context22;

            queryBool.must = (0, _concat["default"])(_context22 = queryBool.must || []).call(_context22, filter);
            return;
          }

          if (filter.missing) {
            var _context23;

            queryBool['must_not'] = (0, _concat["default"])(_context23 = queryBool['must_not'] || []).call(_context23, filter.missing);
            return;
          }

          queryBool.must = (0, _concat["default"])(_context24 = queryBool.must || []).call(_context24, filter);
        });
      } // add optional filters to the query


      if (filters && filters.optional) {
        var _context25;

        (0, _forEach["default"])(_context25 = buildFilters(this, filters.optional)).call(_context25, function (filter) {
          var _context27;

          if (filter.match) {
            var _context26;

            queryBool.should = (0, _concat["default"])(_context26 = (0, _isArray["default"])(queryBool.should) ? queryBool.should : []).call(_context26, filter);
            return;
          }

          queryBool.should = (0, _concat["default"])(_context27 = (0, _isArray["default"])(queryBool.should) ? queryBool.should : []).call(_context27, filter);
        });
      } // add projection


      if (filters && filters.field) {
        var _context28;

        // ensure array
        if (!(0, _isArray["default"])(filters.field)) {
          if (/\,/g.test(filters.field)) {
            filters.field = filters.field.split(/\,/);
          } else {
            filters.field = [filters.field];
          }
        }

        query._source = (0, _concat["default"])(_context28 = query._source || []).call(_context28, filters.field);
      }

      if ((0, _sort["default"])(queryOptions)) {
        if (typeof (0, _sort["default"])(queryOptions) === 'string') {
          var _context29;

          sortOptions = (0, _map["default"])(_context29 = (0, _sort["default"])(queryOptions).split(/\,/g)).call(_context29, function (field) {
            return (0, _trim["default"])(field).call(field);
          });
        }

        if ((0, _isArray["default"])((0, _sort["default"])(queryOptions)) && (0, _sort["default"])(queryOptions).length) {
          sortOptions = (0, _sort["default"])(queryOptions);
        }

        if (_validators["default"].isPOJO((0, _sort["default"])(queryOptions))) {
          var _context30;

          (0, _forEach["default"])(_context30 = (0, _keys["default"])((0, _sort["default"])(queryOptions))).call(_context30, function (property) {
            // support for more traditional way of supplying asc params
            if (property === 'asc') {
              sortFields = (0, _isArray["default"])((0, _sort["default"])(queryOptions).asc) ? (0, _sort["default"])(queryOptions).asc : [(0, _sort["default"])(queryOptions).asc];
              sortOptions = (0, _concat["default"])(sortOptions).call(sortOptions, sortFields);
              return;
            } // support for more traditional way of supplying desc params


            if (property === 'desc') {
              sortFields = (0, _isArray["default"])((0, _sort["default"])(queryOptions).desc) ? (0, _sort["default"])(queryOptions).desc : [(0, _sort["default"])(queryOptions).desc];
              sortFields = (0, _map["default"])(sortFields).call(sortFields, function (field) {
                return '-' + field;
              });
              sortOptions = (0, _concat["default"])(sortOptions).call(sortOptions, sortFields);
              return;
            }

            if ((0, _sort["default"])(queryOptions)[property] === 'desc' || (0, _parseInt2["default"])((0, _sort["default"])(queryOptions)[property], 10) < 0) {
              sortOptions.push('-' + property);
              return;
            }

            sortOptions.push(property);
          });
        }

        query.sort = (0, _sort["default"])(query) || [];
        (0, _forEach["default"])(sortOptions).call(sortOptions, function (field) {
          var sortValue = {};

          if ((0, _startsWith["default"])(field).call(field, '-')) {
            sortValue[field.substring(1)] = 'desc';
          } else {
            sortValue[field] = 'asc';
          }

          (0, _sort["default"])(query).push(sortValue);
        });
      } // add pagination details


      if (typeof queryOptions.start !== 'undefined' && !isNaN(queryOptions.start)) {
        query.from = (0, _parseInt2["default"])(queryOptions.start, 10);
      }

      if (typeof queryOptions.count !== 'undefined' && !isNaN(queryOptions.count)) {
        query.size = (0, _parseInt2["default"])(queryOptions.count, 10);
      } // if no query booleans were applied, remove query


      hasQueryFilters = queryBool.should && queryBool.should.length > 0 || queryBool.must && queryBool.must.length > 0 || queryBool.must_not && queryBool.must_not.length > 0;

      if (!hasQueryFilters) {
        query.query.bool.must = undefined;
      } // if no filter booleans were applied, remove filter


      hasFilterFilters = (0, _filter["default"])(queryBool) && (0, _filter["default"])(queryBool).length > 0;

      if (!hasFilterFilters) {
        query.query.bool.filter = undefined;
      }

      if (!hasFilterFilters && !hasQueryFilters) {
        query.query = defaultQuery;
      } // return to caller


      return query;
    }
  }]);
  return QueryBuilder;
}();

exports.QueryBuilder = QueryBuilder;
var _default = QueryBuilder;
exports["default"] = _default;
//# sourceMappingURL=builder.js.map
